<?php

namespace core\application;
use core\{install\Install, theme\Theme, utility\Utility, user\User, admin\Admin, database\Database, route\Route, translation\Translation as T};

class Application {
	use Form;

	public $result;
	public $trial = false;

	function execute() {
		Route::check_route();
		
		$this->init();
		$this->process();
		$this->render();
		$this->give();

		Route::set_uri_referer();
	}

	function init() {
		define('EQSASH_INTERIM', trim(dirname($_SERVER['PHP_SELF']), '/\\'));

		$settings = Utility::get_settings();
		error_reporting($settings['main']['error_reporting']);
		//set_error_handler('core\utility\Utility::process_errors');
		date_default_timezone_set($settings['main']['timezone']);
		ini_set('session.gc_maxlifetime', $settings['main']['gc_maxlifetime']);
		ini_set('session.cookie_lifetime', $settings['main']['cookie_lifetime']);
		define('EQSASH_INSTALLED', $settings['installed']);
		$this->set_name($settings['main']['name']);
		T::set_language($settings['main']['language']);
		Utility::set_cache($settings['main']['cache']);
		Utility::add_library('window.Eqsash = {apiKey : '.$settings['main']['api_key'].', interim : \''.Route::get_interim_token().'\',}', -1000, 'js');

		$this->set_session($settings['main']['session_name']);
		
		//Database::query_raw(['SET GLOBAL sql_mode = ""'])->execute();
	}
	
	function set_session($session_name) {
		if(empty($_COOKIE[$session_name])) {
			$_COOKIE[$session_name] = $_POST[$session_name] ?? $_GET[$session_name] ?? '';
		}
		
		session_name($session_name);
		session_start();
		
		(new User)->check_authentication();
	}

	//

	function process() {
		EQSASH_INSTALLED ? $this->route() : $this->setup();

		$result = '';
		if (($go_to = Route::get_go_to()) !== false) {
			Utility::set_output_format('data');
			Route::go($go_to);
		} else {
			$buffer = Utility::get_buffer(true);

			$responses = [];
			$result = '';
			foreach ($buffer as $key => $value) {
				$type = $value['type'] ?? 'replace';
				$data = $value['data'] ?? [];

				$response = $value['response'] ?? 'ok';
				$responses[$response][$type] = $type;

				$body = $data['body'] ?? '';
				$result = $type == 'replace' ? $body : $result.$body;
			}

			$error = Utility::get_errors('database');
			$stop = isset($responses['stop']['replace']) || isset($responses['stop']['plus']);

			$not = false;
			if (Utility::get_output_format() == 'theme') {
				if ((isset($responses['not']['replace']) || isset($responses['not']['plus'])) || (!isset($responses['ok']['replace']) && !isset($responses['ok']['plus']))) {
					$not = true;
				}
			}

			$service = $stop ? 'stop' : ($not ? 'not' : '');
			if ($service) {
				$result = Utility::show_service($service);
			}

			$buffer = Utility::get_buffer();
			$buffer['all'] = $result;
			Utility::set_buffer($buffer);
		}

		$this->set_result($result);
	}

	function render() {
		$format = Utility::get_output_format();

		$result = $this->get_result();
		if ($result) {
			if ($format == 'theme') {
				$theme = new Theme;

				$theme_active = '';
				if (EQSASH_INSTALLED) {
					$theme_active = $theme->get_theme('class');
				}

				$result = $theme_active ? (new $theme_active)->show() : $theme->show();
			}
		}

		$this->set_result($result);
	}

	function give() {
		$result = $this->get_result();
		$format = Utility::get_output_format();

		if ($format == 'data') {
			return $result;
		} else {
			echo $result;
		}
	}

	function setup() {
		$buffer = [
			'data' => [
				'body' => (new Install($this))->setup(),
			],
		];

		Utility::add_buffer('install', $buffer);
	}

	function route() {
		$user = new User;
		
		$system = Utility::get_data('system');
		$installation_time = $system['installation_time'] ?? 0;
		
		$uri = Route::get_route();
		$uri_original = Route::get_route(false, null, false);
		$uri_alias = Route::get_route(false, null, false, true);
		$api = Route::check_mode('api');

		if ($this->trial && time() > $installation_time + 60 * 60 * 24 * 7) {
			Utility::add_buffer('admin', ['response' => 'ok', 'type' => 'replace', 'weight' => -1000, 'data' => ['body' => T::get('Время использования системы истекло, для получения полной версии обратитесь к разработчику.')],]);
		} else if ($user->check_stop()) {
			Utility::add_buffer('admin', ['response' => 'stop',]);
		} else if ($api && !Route::check_api()) {
			Route::go_to('');
		} else {
			$routes = Route::get_routes($uri);
			if ($routes && (!$uri_alias || $uri_original == $uri_alias)) {
				foreach ($routes as $key => $value) {
					$title = $value['title'] ?? '';
					$type = $value['type'] ?? 'replace';
					$access = $value['access'] ?? [];
					$entity = $value['entity'];
					$entity_namespace = $value['entity_namespace'];
					$entity_path = $value['entity_path'];
					$handler = $value['handler'];
					$route_original = $value['route_original'];
					$weight = $value['weight'] ?? 0;

					$data = [];
					$stop = false;
					if ($user->get_access($access, $route_original)) {
						if ($title) {
							$this->set_title($title);
						}

						$data = (new $entity_namespace)->$handler();
						if (isset($data['raw']) || isset($data['body'])) {
							if (isset($data['title'])) {
								$this->set_title($data['title']);
							}
						} else {
							$index = $type == 'raw' ? 'raw' : 'body';
							$data = [
								$index => $data,
							];
						}
					} else {
						$stop = true;
					}

					$response = 'ok';
					if ($stop) {
						$response = 'stop';
					} else if (!Utility::clear_array($data)) {
						$response = 'not';
					}

					$buffer = compact('response', 'type', 'weight', 'data');
					Utility::add_buffer($entity, $buffer);
				}
			}

			if ($api) {
				Utility::set_output_format('raw');
			}
		}
	}

	//

	static function set_name($name) {
		$GLOBALS['eqsash']['name'] = $name;
	}

	static function get_name() {
		$name = $GLOBALS['eqsash']['name'] ?? '';

		return $name;
	}

	static function set_title($title) {
		$GLOBALS['eqsash']['title'] = T::get($title);
	}

	static function get_title() {
		$title = $GLOBALS['eqsash']['title'] ?? '';

		return $title;
	}

	function set_result($result) {
		$this->result = $result;
	}

	function get_result() {
		$result = $this->result ?? '';

		return $result;
	}

	//

	function admin() {
		$result = (new Admin)->admin();

		return $result;
	}

	function admin_settings() {
		$settings = Utility::get_settings();
		$settings_main = $settings['main'];

		if (isset($_POST['submit'])) {
			Route::clear_post();

			$settings_main['name'] = $_POST['name'];
			$settings_main['language'] = $_POST['language'] ?: 'ru';
			$settings_main['email_from'] = $_POST['email_from'];
			$settings_main['email_to'] = $_POST['email_to'];
			$settings_main['cache'] = isset($_POST['cache']);

			$settings['main'] = $settings_main;

			T::set_language($_POST['language']);
			Utility::set_settings($settings);
			Utility::clear_cache();
			Utility::set_message('success');
			Route::go_to('admin/all/settings');
		} else {
			$data = [
				'name' => $_POST['name'] ?? $settings_main['name'],
				'language' => $_POST['language'] ?? $settings_main['language'],
				'email_from' => $_POST['email_from'] ?? $settings_main['email_from'],
				'email_to' => $_POST['email_to'] ?? $settings_main['email_to'],
				'cache' => $_POST['cache'] ?? $settings_main['cache'],
			];

			$form = $this->get_form($data);

			return $form;
		}
	}

	function admin_languages() {
		if (isset($_POST['submit'])) {
			Route::clear_post();

			$language = $_POST['language'];
			
			if (!empty($_FILES['strings_source']['tmp_name'][0])) {
				foreach ($_FILES['strings_source']['tmp_name'] as $number => $file_source) {
					$file_target = $_FILES['strings_target']['tmp_name'][$number] ?? '';
					
					if ($file_target) {
						T::import_strings($language, $file_source, $file_target);
						Utility::delete_file($file_target);
					}
					
					Utility::delete_file($file_source);
				}
			} else {
				T::import_strings($language);
			}

			Utility::set_message('success');
			Route::go_to('admin/all/languages');
		} else {
			$form = '<div class="margin-bottom"><a href="/admin/all/languages/export">'.T::get('Скачать строки для перевода').'</a></div>';
			$form .= $this->get_form_languages();

			return $form;
		}
	}

	function admin_languages_export() {
		$dir = EQSASH_ROOT.'/custom/files/all';
		$archive = $dir.'/strings.zip';
		
		//
		
		$mode = Route::get_route(true, 4);
		
		if ($mode == 'hard') {		
			$host = Route::get_host();
			
			$route_aliases = Database::select('route_aliases')
				->data(['id', 'alias',])
				->fetch('assoc_map_all');
				
			$context = stream_context_create([
				'ssl' => [
					'verify_peer' => false,
					'verify_peer_name' => false,
				],
			]);
			
			foreach ($route_aliases as $id => $alias) {
				$route = $alias == '/' ? $alias : '/'.$alias;
				file_get_contents($host.$route, false, $context);
			}
		}
		
		//
		
		$strings = Database::select('strings_source')
			->data(['id', 'source',])
			->fetch('assoc_map_all');
		
		$count = $length = 0;
		$strings_chunk = [];
		foreach ($strings as $id => $source) {
			if ($length > 20000) {
				$length = 0;
				$count++;
			}
			
			$strings_chunk[$count][$id] = $source;
			
			$length += mb_strlen($source);			
		}
		$strings = $strings_chunk;
		
		$files_temp = [];
		foreach ($strings as $key => $strings_item) {
			$file_name = 'strings-'.$key.'.txt';
			$file_path = $dir.'/'.$file_name;
			
			$data = '';
			foreach ($strings_item as $value) {
				$data .= $value.PHP_EOL.PHP_EOL.PHP_EOL;
			}
			$data = rtrim($data).PHP_EOL.PHP_EOL;
		
			file_put_contents($file_path, $data);
			$files_temp[$file_path] = $file_name;
		}
		
		if (count($files_temp) == 1) {
			$result_file = key($files_temp);
		} else {
			Utility::create_archive($archive, $files_temp);	

			foreach ($files_temp as $key => $value) {
				Utility::delete_file($key);
			}
			
			$result_file = $archive;
		}
		
		Utility::download_file($result_file, true, true);
	}

	function admin_languages_clear() {
		Database::delete('strings_source')
			->execute();
		
		Database::delete('strings_target')
			->execute();

		Utility::set_message('success');
		Route::go_to('');
	}

	function admin_reset($redirect = true) {
		Utility::clear_cache();
		Utility::set_message('success');

		if ($redirect) {
			Route::go_to(Route::get_uri_referer(false));
		}
	}
}
