<?php

namespace core\install;
use core\{application\Application, utility\Utility, user\User, route\Route, module\Module, container\Container, translation\Translation as T};

class Install {
	use Api, Form;

	function setup() {
		$result = '';
		if (Route::get_route() != '/') {
			Route::go('');
		} else {
			Route::clear_post();
			Utility::set_settings([], 0700);

			if (isset($_POST['install_database'])) {
				$this->install_database();
			} else if (isset($_POST['install_administrator'])) {
				$this->install_administrator();
			} else {
				$this->install_start();
			}

			$result = $this->result;
		}

		return $result;
	}

	function validate_form($data_validate) {
		extract($data_validate);

		$reg_name = '/^([a-z0-9]{3,30})$/i';
		$reg_password = '/^([_a-z0-9-]{6,20})$/i';
		$reg_email = '/^(([a-z0-9_\.-]+)@([a-z0-9_\.-]+)\.([a-z\.]{2,6}))$/';

		if (!preg_match($reg_name, $user_name)) {
			return T::get('Недопустимое имя пользователя');
		} else if (!preg_match($reg_password, $user_password) || $user_password !== $user_password_confirm || !User::get_password_hash($user_password)) {
			return T::get('Недопустимый пароль или пароли не совпадают');
		} else if (!preg_match($reg_email, $user_email)) {
			return T::get('Недопустимый адрес электронной почты');
		} else {
			return true;
		}
	}

	function install_start() {
		if ($_POST) {
			$this->install_database();
		} else {
			Utility::clear_cache();
			
			Application::set_title('Установка системы: Начало');

			$welcome = T::get('Добро пожаловать');
			$install = T::get('Сейчас Вы сможете произвести установку системы');
			$result = '<h1>'.$welcome.'</h1>'.$install.$this->get_form_start();

			$this->result = $result;
		}
	}

	function install_database() {
		if (isset($_POST['language'])) {
			$settings = Utility::get_settings();
			$settings['main']['language'] = $_POST['language'];
			Utility::set_settings($settings, 0400);
			T::set_language($_POST['language']);
		} else if (isset($_POST['install_database'])) {
			$check = @new \mysqli('localhost', $_POST['database_user_name'], $_POST['database_password'], $_POST['database_name']);
			if ($check->connect_error) {
				Utility::set_message('Неверные данные для подключения к базе данных', 'red');
			} else {
				$settings = Utility::get_settings();
				$settings['database']['name'] = $_POST['database_name'];
				$settings['database']['user_name'] = $_POST['database_user_name'];
				$settings['database']['user_password'] = $_POST['database_password'];
				$settings['database']['host'] = 'localhost';
				$settings['database']['prefix'] = $_POST['database_prefix'];
				Utility::set_settings($settings, 0400);

				$entities_standard = Route::get_entities('', 'namespace', '', false);
				foreach ($entities_standard as $key => $value) {
					$entity = rtrim($value, $key).'Service';

					if (method_exists($entity, 'install')) {
						(new $entity)->install();
					}
				}

				$this->install_administrator();
				return;
			}
		}

		Application::set_title('Установка системы: База данных');

		$data = [
			'database_name' => $_POST['database_name'] ?? '',
			'database_user_name' => $_POST['database_user_name'] ?? '',
			'database_prefix' => $_POST['database_prefix'] ?? '',
		];

		$result = $this->get_form_database($data);

		$this->result = $result;
	}

	function install_administrator() {
		if (isset($_POST['install_administrator'])) {
			$data = [
				'user_name' => $_POST['user_name'],
				'user_password' => $_POST['user_password'],
				'user_password_confirm' => $_POST['user_password_confirm'],
				'user_email' => $_POST['user_email'],
			];

			$validate_result = $this->validate_form($data);
			if ($validate_result !== true) {
				Utility::set_message($validate_result, 'red');
			} else {
				$this->install_final();
				return;
			}
		}

		Application::set_title('Установка системы: Администратор сайта');

		$data = [
			'user_email' => $_POST['user_email'] ?? '',
			'user_name' => $_POST['user_name'] ?? '',
		];

		$result = $this->get_form_administrator($data);

		$this->result = $result;
	}

	function install_final() {
		$user = new User;

		Application::set_title('Установка системы: Завершение');

		$user_name = $_POST['user_name'];
		$user_password = User::get_password_hash($_POST['user_password']);
		$user_email = $_POST['user_email'];
		$token = User::generate_token();

		$user_object = (object)[
			'login' => $user_name,
			'password' => $user_password,
			'email' => $user_email,
			'token' => $token,
			'active' => 1,
			'roles' => ['administrator', 'user',],
		];

		$user_object = $user->save($user_object);

		$user->set_user_id($user_object->id);

		$settings = Utility::get_settings();
		$settings['installed'] = true;
		Utility::set_settings($settings, 0400);

		$data = [
			'install_success' => T::get('Установка системы успешно завершена'),
			'goto_home' => T::get('Вы можете перейти на главную страницу'),
			'http_host' => Route::render_route($_SERVER['HTTP_HOST'], false, true),
			'home_route' => Route::render_route(''),
		];

		$this->result = Utility::build_template(__DIR__.'/templates/final.tpl', $data);
		
		self::register_installation($settings['main']['project'].' '.$settings['main']['project_version'], $settings['main']['api_key']);
		Utility::set_data('system', ['installation_time' => time(),]);
	}
}
