<?php

namespace core\menu;
use core\{utility\Utility, theme\Theme, database\Database, route\Route, translation\Translation as T};

class Menu {
	use Api, Data, Form;
	
	function regenerate() {
		if (T::check_language_change()) {
			$this->make_menu_routes('menu_id');
		}
	}

	function admin() {
		$menus = Database::select('menu')
			->fetch('assoc_all');

		foreach ($menus as $key => $value) {
			$routes_count = Database::select('menu_routes')
				->where('menu_id', $value['id'])
				->fetch('count');

			$value['routes_count'] = $routes_count;
			$menus[$key] = $value;
		}

		$data = [
			'menus' => $menus,
			'add' => T::get('Добавить'),
			'empty' => T::get('Нет данных'),
			'id' => T::get('ID'),
			'name' => T::get('Название'),
			'action' => T::get('Действия'),
			'routes_count' => T::get('Количество ссылок'),
			'list_routes' => T::get('ссылки'),
			'delete' => T::get('удалить'),
			
			'add_route' => Route::render_route('admin/menu/add'),
			'list_routes_route' => Route::render_route('admin/menu/routes'),
			'delete_route' => Route::render_route('admin/menu/delete'),
		];

		$result = (new Theme)->get_template(__DIR__, 'admin', $data);

		return $result;
	}

	function add() {
		$result = '';
		if ($_POST) {
			Route::clear_post();

			$name = $_POST['name'];

			Database::insert('menu')
				->data(['name' => $name])
				->execute();

			Utility::delete_cache('menu');
			Utility::set_message('success');
			Route::go_to('admin/menu');
		} else {
			$result = $this->get_form();
		}

		return $result;
	}

	function delete() {
		$id = Route::get_route(true, 3);

		$result = '';
		if (isset($_POST['submit'])) {
			$menu = Database::select('menu')
				->where('id', $id)
				->fetch('assoc');
			
			Database::delete('menu_routes')
				->where('menu_id', $id)
				->execute();

			$this->set_container($id, [], $menu['entity']);

			Database::delete('menu')
				->where('id', $id)
				->execute();

			Utility::delete_cache('menu');
			Utility::set_message('success');
			Route::go_to('admin/menu');
		} else if (isset($_POST['cancel'])) {
			Route::go_to('admin/menu');
		} else {
			$menu_route = Database::select('menu')
				->where('id', $id)
				->fetch('assoc');

			$result = '';
			if ($menu_route) {
				$result = Utility::get_form_delete();
			}
		}

		return $result;
	}

	function admin_routes() {
		$id = Route::get_route(true, 3);

		$menu = Database::select('menu')
			->where('id', $id)
			->fetch('assoc');

		$result = '';
		if ($menu) {
			$menu_routes = Database::select('menu_routes')
				->where('menu_id', $id)
				->fetch('assoc_all');

			$menu = T::get('Меню');

			$data = [
				'menu' => $menu,
				'menu_id' => $id,
				'menu_routes' => $menu_routes,
				'add' => T::get('Добавить'),
				'empty' => T::get('Нет данных'),
				'id' => T::get('ID'),
				'name' => T::get('Название'),
				'route_title' => T::get('Заголовок'),
				'action' => T::get('Действия'),
				'routes_count' => T::get('Количество ссылок'),
				'route' => T::get('Ссылка'),
				'delete' => T::get('удалить'),
				'edit' => T::get('изменить'),
				
				'add_route' => Route::render_route('admin/menu/routes'),
				'menu_route' => Route::render_route('admin/menu'),
				'edit_route' => Route::render_route('admin/menu/routes/edit'),
				'delete_route' => Route::render_route('admin/menu/routes/delete'),
			];

			$result = (new Theme)->get_template(__DIR__, 'admin_routes', $data);
		}

		return $result;
	}

	function add_route() {
		$id = Route::get_route(true, 3);

		$menu = Database::select('menu')
			->where('id', $id)
			->fetch('assoc');

		$result = '';
		if ($menu) {
			if ($_POST) {
				Route::clear_post();

				$menu_id = $id;
				$title = $_POST['title'];
				$route = $_POST['route'];
				$parent = $_POST['parent'];

				Database::insert('menu_routes')
					->data([
						'menu_id' => $menu_id,
						'title' => $title,
						'route' => $route,
						'parent' => $parent,
					])
					->execute();

				$this->set_container($menu_id, [], $menu['entity']);

				Utility::delete_cache('menu');
				Utility::set_message('success');
				Route::go_to('admin/menu/routes/'.$id);
			} else {
				$data = [
					'title' => '',
					'route' => '',
					'menu_id' => $id,
				];

				$result = $this->get_form_routes($data);
			}
		}

		return $result;
	}

	function edit_route() {
		$id = Route::get_route(true, 4);

		$menu_routes = Database::select('menu_routes')
			->where('id', $id)
			->fetch('assoc');

		$result = '';
		if ($menu_routes) {
			if ($_POST) {
				Route::clear_post();

				$menu_id = $menu_routes['menu_id'];
				$title = $_POST['title'];
				$route = $_POST['route'];
				$parent = $_POST['parent'];

				$menu = Database::select('menu')
					->where('id', $menu_id)
					->fetch('assoc');

				Database::update('menu_routes')
					->data([
						'title' => $title,
						'route' => $route,
						'parent' => $parent,
					])
					->where('id', $id)
					->execute();

				$this->set_container($menu_id, [], $menu['entity']);

				Utility::delete_cache('menu');
				Utility::set_message('success');
				Route::go_to('admin/menu/routes/'.$menu_id);
			} else {
				$data = [
					'title' => $menu_routes['title'],
					'route' => $menu_routes['route'],
					'menu_id' => $menu_routes['menu_id'],
					'parent' => $menu_routes['parent'],
				];

				$result = $this->get_form_routes($data);
			}
		}

		return $result;
	}

	function delete_route() {
		$id = Route::get_route(true, 4);

		$menu_id = Database::select('menu_routes')
			->data(['menu_id'])
			->where('id', $id)
			->fetch('field');

		$result = '';
		if (isset($_POST['submit'])) {
			$menu = Database::select('menu')
				->where('id', $menu_id)
				->fetch('assoc');
			
			Database::delete('menu_routes')
				->where('id', $id)
				->execute();

			$this->set_container($menu_id, [], $menu['entity']);

			Utility::delete_cache('menu');
			Utility::set_message('success');
			Route::go_to('admin/menu/routes/'.$menu_id);
		} else if (isset($_POST['cancel'])) {
			Route::go_to('admin/menu/routes/'.$menu_id);
		} else {
			$result = '';
			if ($menu_id) {
				$result = Utility::get_form_delete();
			}
		}

		return $result;
	}

	//

	function set_container($menu_id, $access = [], $entity = 'menu', $region = '', $cache_clear = false) {
		$body = '';
		
		$entity = $entity ?: 'menu';
		
		$menu_routes = Database::select('menu_routes')
			->where('menu_id', $menu_id)
			->fetch('assoc_all');

		$menu_name = Database::select('menu')
			->data(['name'])
			->where('id', $menu_id)
			->fetch('field');

		if ($menu_routes) {
			foreach ($menu_routes as $key => $value) {
				$body .= '<a href="'.Route::render_route($value['route']).'" title="'.$value['title'].'" class="margin-right-2">'.$value['title'].'</a>';
			}
			
			$data = compact('body');
			
			//

			$container = Database::select('container')
				->where('entity', $entity)
				->where('name', $menu_name)
				->fetch('assoc');

			if (!$container) {			
				$container_in_region = Database::select('container')
					->where('region', 'menu')
					->where('status', 1)
					->fetch('count');

				$region = $region ?: ($container_in_region ? '' : 'menu');
				$access = serialize($access);	
				
				$data += [
					'entity' => $entity,
					'name' => $menu_name,
					'display' => 'all',
					'display_value' => '',
					'access' => $access,
					'region' => $region,
				];

				Database::insert('container')
					->data($data)
					->execute();
			} else {
				Database::update('container')
					->data($data)
					->where('entity', $entity)
					->where('name', $menu_name)
					->execute();
			}
		} else {
			Database::delete('container')
				->where('entity', $entity)
				->where('name', $menu_name)
				->execute();
		}
		
		if ($cache_clear) {
			Utility::delete_cache('menu');
		}
		
		return $body;
	}
}
