<?php

namespace core\user;
use core\{application\Application, database\Database, utility\Utility, form\Form as F, field\Field, route\Route, theme\Theme, translation\Translation as T};

class User {
	use Api, Form;

	private $activate = true;

	function admin() {
		$user = new User;

		$users = $user->load_multi([], true, false);

		if (!$users) {
			$result = T::get('Нет данных');
		} else {
			$roles_names = $this->get_roles_all();

			$result = '<div>
					<a href="'.Route::render_route('user/register').'">'.T::get('Добавить').'</a> |
					<a href="'.Route::render_route('admin/user/roles').'">'.T::get('Роли').'</a> |
					<a href="'.Route::render_route('admin/user/permissions').'">'.T::get('Доступ').'</a>
				</div>
				<table class="admin-table">
					<tr>
						<th>'.T::get('ID').'</th>
						<th>'.T::get('Роли').'</th>
						<th>'.T::get('Логин').'</th>
						<th>'.T::get('Email').'</th>
						<th>'.T::get('Активен').'</th>
						<th>'.T::get('Имя').'</th>
						<th>'.T::get('Создан').'</th>
						<th>'.T::get('Действия').'</th>
					</tr>';

			foreach ($users as $key => $value) {
				$login = '<a href="'.Route::render_route('user/'.$value->id).'">'.$value->login.'</a>';
				$active = T::get($value->active ? 'Да' : 'Нет');
				$created = $value->created ? date('d.m.Y H:i', $value->created) : '';

				$roles = '';
				foreach ($value->roles as $key_two => $value_two) {
					$roles .= '&bull; '.$roles_names[$value_two]->title.'<br>';
				}

				$block = $value->id == 1 ? '' : '<a href="'.Route::render_route('user/block/'.$value->id).'">'.T::get($value->block ? 'разблокировать' : 'блокировать').'</a>';

				$result .= '<tr>
						<td>'.$value->id.'</td>
						<td>'.$roles.'</td>
						<td>'.$login.'</td>
						<td>'.$value->email.'</td>
						<td>'.$active.'</td>
						<td>'.$value->name.'</td>
						<td>'.$created.'</td>
						<td>
							<a href="'.Route::render_route('user/edit/'.$value->id).'">'.T::get('изменить').'</a>
							'.$block.'
							<a href="'.Route::render_route('user/delete/'.$value->id).'">'.T::get('удалить').'</a>
						</td>
					</tr>';
			}
			$result .= '</table>';
		}

		return $result;
	}

	function admin_roles() {
		$user_roles = Database::select('user_role')
			->fetch('object_all');

		if (!$user_roles) {
			$result = T::get('Нет данных');
		} else {
			$result = '<div>
					<a href="'.Route::render_route('admin/user/roles/add').'">'.T::get('Добавить').'</a> |
					<a href="'.Route::render_route('admin/user').'">'.T::get('Пользователи').'</a> |
					<a href="'.Route::render_route('admin/user/permissions').'">'.T::get('Доступ').'</a>
				</div>
				<table class="admin-table">
					<tr>
						<th>'.T::get('ID').'</th>
						<th>'.T::get('Название').'</th>
						<th>'.T::get('Заголовок').'</th>
						<th class="width-30">'.T::get('Действия').'</th>
					</tr>';

			foreach ($user_roles as $key => $value) {
				$result .= '<tr>
						<td>'.$value->id.'</td>
						<td>'.$value->name.'</td>
						<td>'.$value->title.'</td>
						<td>
							<a href="'.Route::render_route('admin/field/list/user/'.$value->id).'">'.T::get('настроить').'</a>
							<a class="margin-left" href="'.Route::render_route('admin/user/roles/edit/'.$value->id).'">'.T::get('изменить').'</a>
							<a class="margin-left" href="'.Route::render_route('admin/user/roles/delete/'.$value->id).'">'.T::get('удалить').'</a>
						</td>
					</tr>';
			}
			$result .= '</table>';
		}

		return $result;
	}

	//

	function get_account() {
		Route::clear_post();

		$user_id = $this->get_id();

		$result = '';
		if (self::check_logged()) {
			Route::go_to('user/'.$user_id);
		} else {
			if (isset($_POST['submit'])) {
				if (isset($_POST['save'])) {
					Utility::set_message('Обнаружены автоматизированные действия', 'red');
					unset($_POST['submit']);
				} else {
					$user_id = $this->authenticate($_POST['login'], $_POST['password']);

					if ($user_id) {
						$this->set_user_id($user_id);
						Route::go_to('user/'.$user_id);
					} else {
						Utility::set_message('Проверьте правильность написания логина и пароля', 'red');
						unset($_POST['submit']);
					}
				}
			}

			$data = [
				'login' => $_POST['login'] ?? '',
			];

			$result = $this->get_form_login($data);
		}

		return $result;
	}

	function logout() {
		$this->set_user_id(0);
		Route::go_to('');
	}

	function add() {
		$user = new User;
		$field = new Field;

		Route::clear_post();

		$mode = $this->get_access('administrator') ? 'admin' : 'default';

		$result = '';
		$raw = [];
		if ($mode == 'default' && $this->check_logged()) {
			Route::go_to('user/'.$this->get_id());
		} else {
			if (isset($_POST['submit'])) {
				$login = $_POST['login'];
				$password = $_POST['password'];
				$confirm_password = $_POST['confirm_password'];
				$email = $_POST['email'];
				$name = $_POST['name'];
				$roles = $this->get_access('administrator') ? self::get_request_group('roles') : ['user',];

				$validate_result = $this->validate(compact('login', 'password', 'confirm_password', 'email', 'name', 'roles'));

				if (!Utility::clear_array($validate_result)) {
					$password_original = $password;
					$password = self::get_password_hash($password);
					$token = self::generate_token();
					$created = time();
					$active = $this->activate ? 0 : 1;

					$user_object = (object)compact('login', 'password', 'email', 'token', 'active', 'name', 'created', 'roles');
					$user_object = $user->save($user_object, false);
					$user_id = $user_object->id;

					$field->get_fields_usage('save', 'user', 1, $user_id);

					Utility::delete_cache('user');

					if ($this->activate) {
						self::send_mail($email, 'mixed', $token, $login, $password_original);
						Utility::set_message('На указанный почтовый ящик выслано письмо с дальнейшими инструкциями', 'green');
					} else {
						Utility::set_message('Регистрация успешно выполнена', 'green');
					}

					if (!$this->check_logged()) {
						$this->set_user_id($user_id);
					}

					$redirect = $mode == 'admin' ? 'admin/user' : 'user/'.$user_id;
					Route::go_to($redirect);
				} else {
					$message = '';
					foreach ($validate_result as $key => $value) {
						$message .= $value ? implode('<br>', $value).'<br>' : '';
					}
					Utility::set_message($message, 'red');
				}
			}

			$data = [
				'login' => $_POST['login'] ?? '',
				'email' => $_POST['email'] ?? '',
				'name' => $_POST['name'] ?? '',
				'roles' => self::get_request_group('roles') ?: ['anonym',],
			];

			$result = $this->get_form($data, 'add', ['anonym',]);
			$raw = $this->get_form($data, 'add', ['anonym',], 0, true);
		}

		$result = [
			'body' => $result,
			'raw' => $raw
		];

		return $result;
	}

	function edit() {
		$user = new User;
		$field = new Field;

		$id = Route::get_route_id();
		$user_id = $this->get_id();
		$user_object_store = $user->load($id, false);

		$result = '';
		$raw = [];
		if ($user_object_store && ($id == $user_id || $this->get_access('administrator'))) {
			Route::clear_post();

			if (isset($_POST['cancel'])) {
				Route::go_to('user/'.$id);
			} else {
				$roles = $user_object_store->roles;

				if (isset($_POST['submit'])) {
					$login = $_POST['login'];
					$password = $_POST['password'];
					$confirm_password = $_POST['confirm_password'];
					$password_old = $_POST['password_old'];
					$password_store = $user_object_store->password;
					$email = $_POST['email'];
					$name = $_POST['name'];
					$roles = $this->get_access('administrator') ? self::get_request_group('roles') : $roles;
					$roles_ids = $this->get_roles_ids($roles);

					$validate_result = $this->validate(compact('login', 'password', 'confirm_password', 'password_old', 'password_store', 'email', 'name') + ['old_login' => $user_object_store->login, 'old_email' => $user_object_store->email, 'roles' => $roles,]);

					if (!Utility::clear_array($validate_result)) {
						$password = $password ? self::get_password_hash($password) : $user_object_store->password;

						$user_object = (object)[
							'id' => $user_object_store->id,
							'login' => $login,
							'password' => $password,
							'email' => $email,
							'name' => $name,
							'roles' => $roles,
						];

						$user_object = $user->save($user_object, false);

						foreach ($roles_ids as $role) {
							$field->get_fields_usage('update', 'user', $role, $id, 0, true);
						}

						Utility::delete_cache('user');
						Utility::set_message('success');
						Route::go_to('user/'.$id);
						return $result;
					} else {
						$message = '';
						foreach ($validate_result as $key => $value) {
							$message .= $value ? implode('<br>', $value).'<br>' : '';
						}
						Utility::set_message($message, 'red');
					}
				}

				$data = [
					'login' => $_POST['login'] ?? $user_object_store->login,
					'email' => $_POST['email'] ?? $user_object_store->email,
					'name' => $_POST['name'] ?? $user_object_store->name,
					'roles' => self::get_request_group('roles') ?: $roles,
					'password_old' => $user_object_store->password,
				];

				$raw = $this->get_form($data, 'edit', $roles, $id, true);
				$result = (new F)->make($raw);
			}
		}

		$result = [
			'body' => $result,
			'raw' => $raw,
		];

		return $result;
	}

	function get_delete() {
		$id = Route::get_route(true, 2);

		$user = Database::select('user')
			->where('id',  $id)
			->fetch('assoc');

		$result = '';
		if ($user) {
			if ($this->get_access('administrator')) {
				$field = new Field;

				if (!$_POST) {
					$result = Utility::get_form_delete();
				} else if (isset($_POST['cancel'])) {
					Route::go_to('admin/user');
				} else {
					$this->delete($id);

					Utility::delete_cache('user');
					Utility::set_message('success');
					Route::go_to('admin/user');
				}
			}
		}

		return $result;
	}

	function show() {
		$user = new User;

		$id = Route::get_route(true, 1);
		$user_id = $this->get_id();

		$user_object = $user->load($id, false);

		$result = '';
		if ($user_object) {
			Application::set_title($user_object->login);

			$link_edit = '';
			if ($user_id) {
				$activate = ($id == $user_id || $this->get_access('administrator')) && $user_object->active == 0;
				if ($activate) {
					Utility::set_message('Данная учетная запись не активирована', 'red');
				}

				$link_edit = $this->get_access('administrator') || $user_object->id == $user_id ? '<a href="'.Route::render_route('user/edit/'.$user_object->id).'">'.T::get('Редактировать').'</a>' : '';
				$link_edit .=  $activate ? ' | <a href="/user/activate/'.$id.'?email='.$user_object->email.'">'.T::get('Активировать').'</a>' : '';
			}

			$field_all = '';
			$roles_ids = $this->get_roles_ids($user_object->roles);
			foreach ($roles_ids as $role) {
				$field_all .= (new Field)->get_fields_usage('show', 'user', $role, $id, 0, true);
			}

			$result = $link_edit.'<div class="user-profile">
					<div class="margin-top margin-bottom">
						<div><i>'.T::get('Имя').'</i>: '.($user_object->name ?: '-').'</div>
						<div><i>'.T::get('Email').'</i>: '.($user_object->email ?: '-').'</div>
						<div><i>'.T::get('Зарегистрирован').'</i>: '.($user_object->created ? date('d.m.Y H:i', $user_object->created) : '-').'</div>
					</div>
					'.$field_all.'
				</div>';
		}

		return $result;
	}

	function validate($data) {
		$reg_login = '/^([a-z0-9]{3,30})$/i';
		$reg_password = '/^([_a-z0-9-]{6,20})$/i';
		$reg_email = '/^(([a-z0-9_\.-]+)@([a-z0-9_\.-]+)\.([a-z\.]{2,6}))$/';
		$reg_name = '/^([а-яёa-z0-9 ]{0,20})$/ui';

		$r_login = Database::select('user')
			->data(['login'])
			->where('login',  $data['login'])
			->fetch('field');

		$r_email = Database::select('user')
			->data(['email'])
			->where('email',  $data['email'])
			->fetch('field');

		$result = [];
		if (!preg_match($reg_login, $data['login']) || ($r_login && (!isset($data['old_login']) || (isset($data['old_login']) && $data['old_login'] != $data['login'])))) {
			$result[] = T::get('Проверьте логин');
		}
		if ((!preg_match($reg_password, $data['password']) && (!isset($data['old_login']) || $data['password'] || $data['confirm_password'])) || $data['password'] !== $data['confirm_password'] || !self::get_password_hash($data['password'])) {
			$result[] = T::get('Проверьте пароль');
		}
		if (!preg_match($reg_email, $data['email']) || ($r_email && (!isset($data['old_email']) || (isset($data['old_email']) && $data['old_email'] != $data['email'])))) {
			$result[] = T::get('Проверьте email');
		}
		if (!preg_match($reg_name, $data['name'])) {
			$result[] = T::get('Проверьте имя');
		}
		if (isset($_POST['save'])) {
			$result[] = T::get('Обнаружены автоматизированные действия');
		}
		if (!empty($data['password_store']) && isset($data['password_old']) && !$this->authenticate($data['login'], $data['password_old'])) {
			$result[] = T::get('Неправильный текущий пароль');
		}

		$fields_all = [];
		$roles_ids = $this->get_roles_ids($data['roles']);
		foreach ($roles_ids as $role) {
			$fields_all_temp = (new Field)->get_fields_usage('validate', 'user', $role);
			$fields_all = array_merge($fields_all_temp, $fields_all);
		}
		$result = array_merge(['body' => $result,], $fields_all);

		return $result;
	}

	function activate() {
		$user_id = Route::get_route(true, 2);
		$email = $_GET['email'] ?? '';

		$result = '';
		if ($email && $user_id) {
			$token = Database::select('user')
				->data(['token',]);
			
			if (!$this->get_access('administrator')) {
				$token->where('id', $this->get_id());
			}
			
			$token = $token->where('email', $email)
				->fetch('field');
			
			if ($token) {
				self::send_mail($email, 'activation', $token);
				Utility::set_message('Данные для подтверждения учетной записи отправлены на ваш email', 'green');
				Route::go_to('user/'.$user_id);
			}
		}

		return $result;
	}

	function set_activation() {
		$token = Route::get_route(true, 2);

		$result_user = Database::select('user')
			->data(['id', 'token', 'active'])
			->where('token', $token)
			->fetch('assoc');

		$result = '';
		if ($result_user && $result_user['active'] == 0) {
			Database::update('user')
				->data(['active' => 1])
				->where('token', $token)
				->execute();

			Utility::set_message('Активация произведена успешно', 'green');
			Route::go_to('user/'.$result_user['id']);
		}

		return $result;
	}

	//

	function add_role() {
		$result = '';
		if ($_POST) {
			Route::clear_post();

			Database::insert('user_role')
				->data([
					'name' => $_POST['name'],
					'title' => $_POST['title'],
				])
				->execute();

			Utility::set_message('success');
			Route::go_to('admin/user/roles');
		} else {
			$data = [
				'name' => '',
				'title' => '',
			];

			$result = $this->get_form_roles($data);
		}

		return $result;
	}

	function edit_role() {
		$id = Route::get_route(true, 4);

		$user_role = Database::select('user_role')
			->where('id',  $id)
			->fetch('assoc');

		$result = '';
		if ($user_role) {
			if ($_POST) {
				Route::clear_post();

				Database::update('user_role')
					->data(['title' => $_POST['title']])
					->where('id',  $id)
					->execute();

				Utility::set_message('success');
				Route::go_to('admin/user/roles');
			} else {
				$data = [
					'name' => $user_role['name'],
					'title' => $user_role['title'],
				];

				$result = $this->get_form_roles($data, 'edit');
			}
		}

		return $result;
	}

	function delete_role() {
		$id = Route::get_route(true, 4);

		$user_role = Database::select('user_role')
			->where('id',  $id)
			->fetch('assoc');

		$result = '';
		if ($user_role) {
			if (isset($_POST['submit'])) {
				Database::delete('user_role')
					->where('id', $id)
					->execute();

				Database::delete('user_access')
					->where('role_id', $id)
					->execute();

				Utility::set_message('success');
				Route::go_to('admin/user/roles');
			} else if (isset($_POST['cancel'])) {
				Route::go_to('admin/user/roles');
			} else {
				$result = Utility::get_form_delete();
			}
		}

		return $result;
	}

	function reset_permissions() {
		Database::delete('user_access')->execute();

		Utility::set_message('success');
		Route::go_to('admin/user/permissions');
	}

	function set_permissions() {
		$routes = Route::get_routes_all();
		$user_roles = $this->get_roles_all();

		$user_roles_all = [];
		foreach ($user_roles as $key => $value) {
			$user_roles_all[] = $value->name;
		}

		$user_access = [];
		foreach ($routes as $key => $value) {
			foreach ($value as $key_two => $value_two) {
				$access_temp = $value_two['access'] ?? $user_roles_all;

				foreach ($access_temp as $key_three => $value_three) {
					$role_id = $user_roles[$value_three]->id;
					$user_access[$key_two][$role_id] = $role_id;
				}
			}
		}

		$map = [];
		foreach ($user_access as $key => $value) {
			$map[] = $key;
		}

		$result = '';
		if ($_POST) {
			unset($_POST['submit']);

			Route::clear_post();

			Database::delete('user_access')->execute();

			$access = [];
			foreach ($_POST as $key => $value) {
				$string_parts = Utility::get_string_parts($key, '_');

				$access[$map[$string_parts[0]]][] = $string_parts[1];
			}

			foreach ($access as $key => $value) {
				foreach ($value as $key_two => $value_two) {
					Database::insert('user_access')
						->data(['route' => $key, 'role_id' => $value_two])
						->execute();
				}
			}

			Utility::set_message('success');
			Route::go_to('admin/user/permissions');
		} else {
			if (!$user_access) {
				$result = T::get('Нет данных');
			} else {
				$user_access_data = Database::select('user_access')
					->fetch('assoc_all');

				$user_access_save = [];
				foreach ($user_access_data as $key => $value) {
					$user_access_save[$value['route']][] = $value['role_id'];
				}

				$user_roles_th = '';
				foreach ($user_roles as $key => $value) {
					$user_roles_th .= '<th class="text-center">'.$value->title.'</th>';
				}

				$result = '<table class="admin-table">
					<tr>
						<th>'.T::get('Путь').'</th>
						'.$user_roles_th.'
					</tr>';

				$form = new F;

				foreach ($user_access as $key => $value) {
					$value = $user_access_save[$key] ?? ($user_access_save ? [] : $value);

					$roles = '';
					foreach ($user_roles as $key_two => $value_two) {
						$checkbox = $form->make_field([
							[
								'input' => [
									'type' => 'checkbox',
									'name' => array_search($key, $map).'_'.$value_two->id,
								],
								'checked' => in_array($value_two->id, $value),
							],
						]);

						$roles .= '<td class="text-center">'.$checkbox.'</td>';
					}

					$result .= '<tr><td>'.$key.'</td>'.$roles.'</tr>';
				}
				$result .= '</table>';

				$form = $form->make($result, ['submit'], false);

				$result = '<div>
						<a href="'.Route::render_route('admin/user/permissions/reset').'">'.T::get('Сбросить').'</a> |
						<a href="'.Route::render_route('admin/user').'">'.T::get('Пользователи').'</a> |
						<a href="'.Route::render_route('admin/user/roles').'">'.T::get('Роли').'</a>
					</div>'.$form;
			}
		}

		return $result;
	}
	
	//

	function admin_stop() {
		$users_stop = Utility::get_data('users_stop');
		
		$result = T::get('Нет данных');
		if ($users_stop) {
			$result = '';
			foreach ($users_stop as $key => $value) {
				$data = [];
				$data['ip'] = $key;
				$data['time'] = date('d.m.Y H:i', $value);
				$data['action'] = T::get('Разблокировать');
				$users_stop[$key] = $data;
			}
			
			$result .= (new Theme)->get_template(__DIR__, 'admin_stop', compact('users_stop'));
		}
		
		$result = '<div><a href="/admin/user/stop/add">'.T::get('Добавить').'</a></div>'.$result;
		
		return $result;
	}

	function admin_stop_add() {
		if (isset($_POST['submit'])) {
			$ip = $_POST['ip'] ?? 0;
		
			Api::set_stop($ip);
		
			Utility::set_message('success');
			Route::go_to('admin/user/stop');
		} else {		
			$result = self::get_form_stop();
			
			return $result;
		}
	}

	function admin_stop_delete() {
		$number = Route::get_route_id();
		
		$users_stop = Utility::get_data('users_stop');
		unset($users_stop[$number]);
		Utility::set_data('users_stop', $users_stop);
		
		Utility::set_message('success');
		Route::go_to('admin/user/stop');
	}

	function block() {
		$user_id = Route::get_route_id();
	
		if ($user_id != 1) {
			$user = Database::select('user')
				->where('id', $user_id)
				->fetch('object');

			if ($user) {
				$block = !$user->block;

				Database::update('user')
					->data(compact('block'))
					->where('id', $user_id)
					->execute();
			}
		}
		
		Utility::set_message('success');
		Route::go_to('admin/user');
	}
}
